# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""usage: as_run --serv [options] user@mach:/nom_profil.export

old syntax : as_serv as_exec [options] user@mach:/nom_profil.export

Export file must be the last argument.

Object :
   - retreive the export file from the client
   - read export and ...
   - ... call as_run on the right node, into a xterm or not,
     now or differed...
"""

__revision__ = "$Id: as_serv.py 3301 2008-04-21 13:57:01Z courtois $"

import os
import sys
import re
import getpass

from as_job    import nodepara
from as_profil import ASTER_PROFIL

# ----- differ messages translation
def _(mesg):
   return mesg


#-------------------------------------------------------------------------------
def SetParser(run):
   """Configure the command-line parser, add options name to store to the list,
   set actions informations.
   run : ASTER_RUN object which manages the execution
   """
   acts_descr = {
      'serv' : {
         'method' : Serv,
         'syntax' : 'user@mach:/nom_profil.export',
         'help'   : 'start an execution (calling as_run in a separate process)'
      },
   }
   run.SetActions(
         actions_descr = acts_descr,
         actions_order = ['serv',],
         group_options=False,
         options_descr = {},
   )

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
msg_pb_version = """######################################################################

WARNING : ASTK Server version %s differs from
          ASTK Client version %s.
          An update from Server or Client is necessary.
          This execution may fail ! 
          If it works results should be correct.

######################################################################
"""

msg_info = """###############################################
           Client name : %(mcli)s
              Username : %(ucli)s
%(sep)s
           Server name : %(serv)s
              Username : %(user)s
                  Node : %(node)s
              Platform : %(plt)s
%(sep)s
    Code_Aster version : %(vers)s
%(sep)s
            Time (min) : %(tpsjob)s
           Memory (MB) : %(vmem).1f
  Number of processors : %(nbp)s   (OpenMP)
       Number of nodes : %(mpi_nbnoeud)s   (MPI)
  Number of processors : %(mpi_nbcpu)s   (MPI)
%(sep)s
            Debug mode : %(dbg)s"""

msg_classe  = " specified batch queue : %s"
msg_depart  = "            start time : %s"
msg_consbtc = "            BTC script : %s"
msg_vers  = """   Version ASTK Server : %s
   Version ASTK Client : %s
###############################################
"""

SEPAR = '%NEXT%'

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class AS_SERV:
   """This class read user's profile and (if needed) call as_run through or not
   a terminal, or just write a btc file...
   """
   _supported_services = ('meshtool', 'stanley', 'convbase')

   def __init__(self, run, filename):
      """Initialisations"""
      self.run = run
      self.jobid       = self.run['num_job']

      # ----- profile filename
      fprof = os.path.join(self.run['tmp_user'], 'profil_astk.%s' % self.jobid)
      self.run.to_delete.append(fprof)
      
      # ----- read profile from args
      kret = self.run.Copy(fprof, filename, niverr=_('<F>_PROFILE_COPY'))
      self.prof = ASTER_PROFIL(fprof, self.run)

      # ----- decode service called
      serv = self.prof['special'][0].split(SEPAR)[0]
      if serv != '':
         self.prof = self.decode_special_service(serv)

      # attributes
      self.dict_info   = None
      self.as_exec_ref = os.path.join(self.run.astk_serv_root, 'bin', 'as_exec_ref')
      self.nodepara    = nodepara

#-------------------------------------------------------------------------------
   def error(self, msg):
      """Print an error msg and exit.
      """
      self.run.Mess(msg, '<F>_ERROR')

#-------------------------------------------------------------------------------
   def decode_special_service(self, serv):
      """Return the profile modified for the "special" service.
      """
      if serv not in self._supported_services:
         self.error(_('Unknown service : %s') % serv)      
      
      modifier = ModifierFactory(serv, self.prof, self.run)
      modifier.modify()
      return modifier.return_profile()

#-------------------------------------------------------------------------------
   def on_mach_ref(self):
      """Are we on reference server ?
      """
      return os.path.exists(self.as_exec_ref) and os.access(self.as_exec_ref, os.X_OK)

#-------------------------------------------------------------------------------
   def flash(self, typ, num_job=None):
      """If typ='o', return something like .../flasheur/nomjob.o1234
      """
      assert self.prof is not None
      if num_job is None:
         num_job = self.jobid
      return os.path.join(self.run['flasheur'],
            '%s.%s%s' % (self.prof['nomjob'][0], typ, num_job))

#-------------------------------------------------------------------------------
   def build_dict_info(self, opts):
      """Build a dictionnary grouping all parameters.
      """
      sep = "-----------------------------------------------"
      mode = self.prof['mode'][0]
      node = self.prof['noeud'][0] or self.prof['serveur'][0]
      self.dict_info = {
         'sep'                : sep,
         'export'             : self.prof.filename,
         'mcli'               : self.prof['mclient'][0],
         'ucli'               : self.prof['uclient'][0],
         'serv'               : self.prof['serveur'][0],
         'user'               : self.prof['username'][0],
         'mode'               : mode,
         'node'               : node,
         'plt'                : self.prof['plate-forme'][0],
         'vers'               : self.prof['version'][0],
         'tpsjob'             : self.prof['tpsjob'][0],
         'vmem'               : int(self.prof['memjob'][0]) / 1024.,
         'nbp'                : self.prof['ncpus'][0],
         'mpi_nbnoeud'        : self.prof['mpi_nbnoeud'][0],
         'mpi_nbcpu'          : self.prof['mpi_nbcpu'][0],
         'dbg'                : self.prof['debug'][0],
         'prof_content'       : self.prof.content,
         'nomjob'             : self.prof['nomjob'][0],
         'nomjob_'            : self.flash('', ''),
         'nomjob_p'           : self.flash('p', '$num_job'),
         'sbin'               : os.path.join(self.run.astk_serv_root, 'bin'),
         'slib'               : os.path.join(self.run.astk_serv_root, 'lib'),
         'who'                : getpass.getuser(),
         'opts'               : opts,
      }
      
      if self.prof['srv_dbg'][0] == 'oui':
         self.dict_info['opts'] += ' --debug'
      if self.prof['srv_verb'][0] == 'oui':
         self.dict_info['opts'] += ' --verbose'
      
      # rep_trav from profile or config(_nodename)
      rep_trav = self.prof['rep_trav'][0]
      if rep_trav == '':
         rep_trav = self.nodepara(self.run, node, 'rep_trav')
      self.dict_info['rep_trav'] = rep_trav
      # set message using previous content
      self.dict_info['message'] = self.message()

#-------------------------------------------------------------------------------
   def message(self):
      """Format information message.
      """
      # No "' in ASTK_MESSAGE !
      ASTK_MESSAGE = []

      # check client and server versions
      serv_vers = self.run.__version__
      try:
         client_vers = self.prof['origine'][0].split()[1]
      except Exception, msg:
         self.run.DBG('Error : unexpected "origine" value :', self.prof['origine'][0])
         client_vers = ''
      if serv_vers != client_vers:
         ASTK_MESSAGE.append(msg_pb_version % (serv_vers, client_vers))

      ASTK_MESSAGE.append(msg_info % self.dict_info)

      if self.prof['classe'][0]:
         ASTK_MESSAGE.append(msg_classe % self.prof['classe'][0])
      if self.prof['depart'][0]:
         ASTK_MESSAGE.append(msg_depart % self.prof['depart'][0])
      ASTK_MESSAGE.append(self.dict_info['sep'])

      if self.prof['consbtc'][0] == 'oui':
         msg = "generated"
      else:
         msg = "provided by user"
      ASTK_MESSAGE.append(msg_consbtc % msg)
      ASTK_MESSAGE.append(self.dict_info['sep'])

      ASTK_MESSAGE.append(msg_vers % (serv_vers, client_vers))

      return os.linesep.join(ASTK_MESSAGE)

#-------------------------------------------------------------------------------
   def get_jobid(self, txt):
      """Search for JOBID=xxxx"""
      mat = re.search('JOBID[ ]*=[ ]*([0-9]+)', txt)
      if mat is not None:
         jobid = mat.group(1)
      else:
         jobid = '?'
      return jobid

#-------------------------------------------------------------------------------
   def consbtc(self, fbtc):
      """Write btc file.
      """
      assert type(self.dict_info) is dict
      
      mode = self.prof['mode'][0]

      if self.prof['display'][0]:
         str_display = 'DISPLAY=%s ; export DISPLAY' % self.prof['display'][0]
      else:
         str_display = ''
      aster_profile = os.path.join(self.run.astk_serv_root, 'conf', 'aster_profile.sh')
      if os.path.exists(aster_profile):
         str_aster_profile = '. %s' % aster_profile
      else:
         str_aster_profile = ''
      if mode == 'interactif':
         m = 'i'
         str_jobid = 'num_job=%s' % self.jobid
      else:
         m = 'b'
         str_jobid = """echo $%s | awk '{inv=$0; sub("^[0-9]+","",inv); sub(inv,"",$0); print $0;}'""" % self.run['batch_jid']

      btc = """#!/bin/sh

ASTER_VERSION=%(vers)s
export ASTER_VERSION

%(str_display)s
%(str_aster_profile)s
%(str_jobid)s

# copie du .export dans le flasheur et rep_trav
cat << EOFEXPORT > %(nomjob_p)s
%(prof_content)s
EOFEXPORT

# on redéfinit car déjà arrivé en dhcp ou quand on change de noeud...
LOGNAME=%(who)s
export LOGNAME

# message d'info
printf %(message)r

# lance l'exec
%(sbin)s/as_run %(opts)s %(nomjob_p)s --num_job=$num_job | tee %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s

# diagnostic
egrep -- '--- DIAGNOSTIC JOB :' %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s | awk '{print $5}' > %(nomjob_)s%(m)s$num_job
\\rm -f %(rep_trav)s/%(nomjob)s.$num_job.fort.6.%(m)s

\\rm -rf %(fbtc)s

"""
      dict_btc = self.dict_info.copy()
      dict_btc.update({
         'str_display'       : str_display,
         'str_aster_profile' : str_aster_profile,
         'str_jobid'         : str_jobid,
         'm'                 : m,
         'fbtc'              : fbtc,
      })

      open(fbtc, 'w').write(btc % dict_btc)
      os.chmod(fbtc, 0755)

#-------------------------------------------------------------------------------
   def soumbtc_batch(self, fbtc):
      """Run btc script in batch mode.
      """
      dict_btc = self.dict_info.copy()
      dict_btc['fbtc'] = fbtc
      cmd = '%(slib)s/sub_script %(export)s %(fbtc)s' % dict_btc
      iret, output = self.run.Shell(cmd)
      if iret != 0:
         print output
         print _("Error during submitting job !")
         self.run.Sortie(5)
      # decode output
      mat = re.search('JOBID=(.*) QUEUE=(.*)', output)
      if mat is None:
         print _("Unexpected output : %") % output
         self.run.Sortie(5)
      else:
         JOBID, QUEUE = [s.strip() for s in mat.groups()]
      return JOBID, QUEUE

#-------------------------------------------------------------------------------
   def soumbtc_interactif(self, fbtc):
      """Run btc in interactive mode.
      """
      # commandes
      cmd_batch      = '%(fbtc)s 1> %(output)s 2> %(error)s'
      cmd_interactif = '%(fbtc)s 2> %(error)s | tee %(output)s'

      node = self.dict_info['node']
      dico = {
         'fbtc'   : fbtc,
         'output' : self.flash('o'),
         'error'  : self.flash('e'),
      }
      # follow output or not
      xterm = self.prof['xterm'][0]
      if self.prof['depart'][0] != '' or xterm == '':
         cmd =  cmd_batch % dico
      else:
         cmd = cmd_interactif % dico

      # delayed start
      if self.prof['depart'][0] != '':
         cmd = "echo '%s' | at %s" % (cmd, self.prof['depart'][0])
      elif xterm != '':
         cmd = '%s -e /bin/sh -c "%s"' % (xterm, cmd)
      
      # run on another node
      distant = self.run.GetHostName().split('.')[0] != node.split('.')[0]
      
      if not distant:
         # check xterm command
         if xterm != '':
            term = xterm.split()[0]
            if not os.access(term, os.X_OK):
               print _("Not an executable : %s") % term
               self.run.Sortie(7)
      else:
         # check node connection
         iret, output = self.run.Shell('echo hello', mach=node)
         if output.find('hello') < 0:
            print output
            print _("Connection failure to %s (from %s)") % (node, self.prof['serveur'][0])
            self.run.Sortie(6)

      iret, output = self.run.Shell(cmd, mach=node, bg=True)

      return self.jobid, 'interactif'

#-------------------------------------------------------------------------------
   def start(self, options=''):
      """Go !
      """
      # ----- copy and read .export, build dict for formatting
      self.build_dict_info(options)

      jn = self.jobid
      mode = self.prof['mode'][0]
      JOBID = -1
      QUEUE = "unknown"

      # ----- batch + machref, call as_exec_ref
      if mode == 'batch' and self.on_mach_ref():
         fbtc = os.path.join(self.run['tmp_user'], '%s.u%s' % (self.prof['nomjob'][0], jn))
         os.environ['ASTK_MESSAGE'] = repr(self.dict_info['message'])
#         os.environ['ASTK_MESSAGE'] = 'Unable to print ASTK_MESSAGE !'
         
         cmd = self.as_exec_ref + ' ' + self.prof.filename + ' ' + fbtc
         iret, output = self.run.Shell(cmd, follow_output=True)
         
         self.run.DBG('OUTPUT of as_exec_ref :', output, all=True)
         
         self.jobid = self.get_jobid(output)
         if os.path.exists(fbtc):
            jret = self.run.Rename(fbtc, self.flash('u'))
         jret = self.run.Copy(self.flash('p'), self.prof.filename)
         return iret, output

      # ----- consbtc ?
      fbtc = os.path.join(self.run['flasheur'], 'btc.%s' % jn)
      if self.prof['consbtc'][0] == 'non' and not 'make_env' in self.prof['actions']:
         fbtc0 = self.prof.Get('D', 'btc')[0]['path']
         iret = self.run.Copy(fbtc, fbtc0)
      else:
         self.consbtc(fbtc)

      # ----- soumbtc ?
      if self.prof['soumbtc'][0] == 'oui':
         if mode == 'interactif':
            JOBID, QUEUE = self.soumbtc_interactif(fbtc)
         else:
            JOBID, QUEUE = self.soumbtc_batch(fbtc)

         # copy fbtc into flasheur/
         self.run.Copy(self.flash('u'), fbtc)

      # return job info
      print "JOBID=%s QUEUE=%s" % (JOBID, QUEUE)

      # faut-il recopier le btc vers le client
      res_fbtc = self.prof.Get('R', 'btc')
      if len(res_fbtc) > 0:
         res_fbtc = res_fbtc[0]['path']
         self.run.Copy(res_fbtc, fbtc)

      return 0, ''


#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class ProfileModifier(object):
   """ ;-) """
   service = 'service_name'
   
   def __init__(self, prof, run=None):
      """Init."""
      self.run       = run
      self.prof_orig = prof
      self.new_prof  = self.prof_orig.copy()
      self.special   = self.prof_orig['special'][0].split(SEPAR)

   def modify(self):
      """Modifications for all services.
      """
      # keep arguments
      # empty data and result
      self.new_prof.data = []
      self.new_prof.resu = []
      
      # set some parameters
      self.new_prof['nomjob']  = '%s_%s' % (self.prof_orig['nomjob'][0], self.service)
      self.new_prof['actions'] = 'make_etude'
      self.new_prof['ncpus']   = 1
      self.new_prof['consbtc'] = 'oui'
      self.new_prof['soumbtc'] = 'oui'
      
      # remove some parameters :
      for p in ('special',):
         del self.new_prof[p]

   def filename(self, filename):
      """If we start execution on a remote node, we must add user@'here'."""
      node = self.prof_orig['noeud'][0].split('.')[0]  # without domain name
      user, host = self.run.system.getuser_host()
      if node != host:
         filename = '%s@%s:%s' % (user, host, filename)
      return filename
   
   def return_profile(self):
      return self.new_prof

#-------------------------------------------------------------------------------
class ProfileModifierMeshtool(ProfileModifier):
   """Modifier for meshtool service.
   """
   service = 'meshtool'

   def modify(self):
      """Modifications for meshtool service.
      """
      super(ProfileModifierMeshtool, self).modify()
      
      # job name
      self.new_prof['nomjob']  = '%s_mesh' % self.prof_orig['nomjob'][0]
      
      # commands file
      fcomm = os.path.join(self.run.astk_serv_root, 'lib', 'meshtool.comm')
      self.new_prof.Set('D',
         { 'path' : fcomm, 'ul' : 1, 'type' : 'comm',
           'isrep' : False, 'compr' : False})

      # special : mesh IN (unit 71), mesh OUT (unit 72)
      assert len(self.special) >= 3
      self.new_prof.parse("""%s 71\n%s 72""" % tuple(self.special[1:3]))

      # parameter file : field 4 and next
      fpara = os.path.join(self.run['tmp_user'], 'meshtool.%s.para' % self.run['num_job'])
      self.run.Delete(fpara)
      txt = os.linesep.join(self.special[3:])
      open(fpara, 'w').write(txt)
      self.new_prof.Set('D',
         { 'path' : self.filename(fpara), 'ul' : 70, 'type' : 'libr',
           'isrep' : False, 'compr' : False})
      
#-------------------------------------------------------------------------------
class ProfileModifierConvbase(ProfileModifier):
   """Modifier for convbase service.
   """
   service = 'convbase'

   def modify(self):
      """Modifications for convbase service.
      """
      super(ProfileModifierConvbase, self).modify()
      
      # commands file
      fcomm = os.path.join(self.run.astk_serv_root, 'lib', 'convbase.comm')
      self.new_prof.Set('D',
         { 'path' : fcomm, 'ul' : 1, 'type' : 'comm',
           'isrep' : False, 'compr' : False})

      # special : base IN, base OUT
      assert len(self.special) >= 3
      self.new_prof.parse("""%s\n%s""" % tuple(self.special[1:3]))
      

#-------------------------------------------------------------------------------
class ProfileModifierStanley(ProfileModifier):
   """Modifier for stanley service.
   """
   service = 'stanley'

   def modify(self):
      """Modifications for stanley service.
      """
      super(ProfileModifierStanley, self).modify()
      
      # job name
      self.new_prof['nomjob']  = '%s_post' % self.prof_orig['nomjob'][0]
      
      # commands file
      fcomm = os.path.join(self.run.astk_serv_root, 'lib', 'stanley_post.comm')
      self.new_prof.Set('D',
         { 'path' : fcomm, 'ul' : 1, 'type' : 'comm', 'isrep' : False, 'compr' : False})

      # special : "R base repertoire flag 0"
      assert len(self.special) >= 2
      self.new_prof.parse(self.special[1])

#-------------------------------------------------------------------------------
def ModifierFactory(service, prof, run=None):
   if service == 'meshtool':
      return ProfileModifierMeshtool(prof, run)
   elif service == 'stanley':
      return ProfileModifierStanley(prof, run)
   elif service == 'convbase':
      return ProfileModifierConvbase(prof, run)
   return None


#-------------------------------------------------------------------------------
def Serv(run, *args):
   """Start an execution, another as_run, in a separate process.
   """
   # ----- check argument
   if len(args) != 1:
      run.parser.error(_("'--%s' requires one argument") % run.current_action)

   service = AS_SERV(run, args[0])
   iret, output = service.start()
   run.Sortie(iret)



