# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""This module defines the Dispatcher class.
"""

__revision__ = "$Id: as_build.py 3309 2008-06-05 08:49:38Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
from Queue import Queue, Empty

try:
   import threading as _threading
except ImportError:
   import dummy_threading as _threading

#-------------------------------------------------------------------------------
class Task:
   """Task object (will be dispatched in different thread).
   """
   def __init__(self, **kwargs):
      """Constructor - initialization : store kwargs items as attributes.
      Attributes 'OUT' should also be initialized."""
      for k, v in kwargs.items():
         if hasattr(self, k):
            raise AttributeError, "'%s' attributes already exists !" % k
         setattr(self, k, v)
   
   def execute(self, item):
      """Function called for each item of the stack.
      Warning : 'execute' should not modify attributes.
      """
      raise NotImplementedError, 'must be overridden in a subclass'

   def result(self, **kwargs):
      """Function called after each task to treat results of execute.
      Arguments are 'execute' results + keywords args.
      'result' is called thread-safely, so can store results in attributes.
      """
      raise NotImplementedError, 'must be overridden in a subclass'

#-------------------------------------------------------------------------------
class Dispatcher:
   """Execute a function in parallel.
   """
   result_lock = None
   
   def __init__(self, l_stack, task, numthread=1):
      """Execute in separated thread (parallel) :
      
         for value in l_stack:
            out = task.execute(value)
            task.result(out) # vars_out allows to store "global" result.
      """
      queue = Queue()                  # job queue
      Worker.numthread = 0             # thread creation count
      self.l_thread = []               # threads
      Dispatcher.result_lock = _threading.Lock()
      
      assert isinstance(task, Task), "'task' argument must be a Task instance !"
      
      # fill the queue
      for item in l_stack:
         queue.put(item)
      # spawn threads
      for i in range(numthread):
         t = Worker(queue, task)
         self.l_thread.append(t)
         t.start()
      # wait for threads to finish
      for t in self.l_thread:
         t.join()
   
   def store_result(func, args, **kwargs):
      """Store/print result from threads.
      """
      Dispatcher.result_lock.acquire()
      try:
         func(*args, **kwargs)
      except Exception, err:
         print 'EXCEPTION (task.result) : %s' % err
      Dispatcher.result_lock.release()
   store_result = staticmethod(store_result)
   
   def report(self):
      """Report.
      """
      tot = 0
      occ = []
      txt = []
      for t in self.l_thread:
         thr_id, nb = t.report()
         tot += nb
         occ.append((thr_id, nb))
      for thr_id, nb in occ:
         txt.append('Thread %3d visited %6d times - %3d%%' % (thr_id, nb, int(100.*nb/tot)))
      txt.insert(0, 'Dispatcher report - %6d calls' % tot)
      return os.linesep.join(txt)

#-------------------------------------------------------------------------------
class Worker(_threading.Thread):
   """Worker thread for parallel execution.
   """
   numthread = 0                        # thread creation count
   
   def __init__(self, queue, task):
      self.queue = queue                # Queue of sorting jobs to do
      Worker.numthread += 1             # update count of created threads
      self.threadid = self.numthread    # unique id of this thread
      self.loop = 0                     # work done by thread
      self.task = task
      _threading.Thread.__init__(self)
      
   def run(self):
      """Thread loops taking jobs from queue until none are left.
      """
      while True:
         try:
            # get job, Queue handles the locks for us
            item = self.queue.get(False)
         except Empty:
            break
         
         try:
            # call the function
            result = self.task.execute(item)
         except Exception, err:
            print 'EXCEPTION (task.execute) : %s' % err
            result = None
         Dispatcher.store_result(self.task.result, result, threadid=self.threadid)
         self.loop += 1
      
   def report(self):
      return (self.threadid, self.loop)


